#ifndef CC_LIST_H
#define CC_LIST_H
//
// Die OOP-Definition einer 'vollstndig unabhngigen' Liste.
//
// 12.03.1992 Andre	Idee, erste Geanken, gelesen (RKM_EXEC)
// 13.03.1992 Andre	ein erster Versuch
//

// Nur unter AmigaOS Intuition und die Displayfunktionen zugnglich machen
#ifdef amigados
#include <exec/types.h>
#include <exec/lists.h>
#include <intuition/intuition.h>
#include <intuition/gadgetclass.h>
#include <libraries/gadtools.h>
#include <graphics/displayinfo.h>

// Weiterhin *mu* irgendwo im entsprechende Hauptprogramm IntuitionBase
// und GadToolsBase deklariert und *initialisiert* sein
extern struct IntuitionBase *IntuitionBase;
extern struct GadToolsBase  *GadToolsBase;

// Auerdem mssen noch die beiden nchsten Variablen deklariert und
// initialisiert sein:
extern struct Screen	    *theScreen;
extern APTR		     VisualInfo;
// Diese Initialisierung wird i.a. folgendermassen Durchgefhrt:
//
//  if ( NOT( theScreen = LockPubScreen( "Workbench" )))  - oder anderen Namen
//    error(...);
//  if ( NOT( VisualInfo = GetVisualInfo( theScreen, TAG_DONE )))
//    error(...);
#endif

enum copyway
{
   nocopy, copy
};

enum dsply
{
   readonly, readwrite
};

class node
{
   node 	 *succ;      // Eine doppelt verkettete Liste
   node 	 *pred;
   long 	  pri;	     // Die Prioritt des Elements (klein: unwichtig)
   char 	 *name;      // Der Name
   unsigned long  pack;      // Gepackte vier Byte als Kurznamen
 public:
			     // Der Konstruktor
   node(char *n, copyway cw=copy, long pr=0, unsigned long pa=0);
   ~node(void);              // Der Destruktor

			     // Methoden der Liste

   node *getsucc(void);      // Holt den Zeiger auf den nchsten Knoten
   node *getpred(void);      // Holt den Zeiger auf den Vorgnger

   void setsucc(node *s);    // Setzt den Nachfolger auf s
   void setpred(node *p);    // Setzt den Vorgaenger auf p

			     // Methoden der Daten

   long getpri(void);        // Liet die Prioritt des Knotens
   char *getname(void);      // Liet den Namen des Knotens
   unsigned long getpack(void); // Liet den gepackten Namen des Knotens

   void setpri(long p);      // Setzt die Prioritt des Knotens neu
   void setname(char *n, copyway cw=copy); // Setzt den Namen neu
			     // wenn cw auf 'copy' gesetzt ist (was
			     // Voreinstellung ist), wird der Name vor der
			     // Verwendung kopiert. Bei 'nocopy' wird das
			     // Kopieren unterlassen.
   void setpack(unsigned long pa);  // Setzt den gepackten Namen neu
   void setpackchar(char *p); // Setzt den gepackten Namen neu, und zwar
			     // werden dazu die ersten vier Zeichen des
			     // Strings gepackt. Ist der String krzer als
			     // vier Zeichen, so wird mit Nullen aufgefllt.

   // Die Funktionen zum Ausgeben
      void print(void);
};


// Und die Klasse der Listen

class list
{
   node     head;	     // Der vordere Anker
   node     tail;	     // Der hintere Anker
 public:
   list(void);               // Der Konstruktor
   ~list(void);              // Der Destruktor

   node *getfirst(void);
   node *getlast(void);

   // Allgemeine Komanndos der Liste

   void insert(node *p, node *the);  // Kettet 'the' nach 'p' in die
			     // Liste ein.
   void remove(node *n);     // Der Knoten 'n' wird aus der Liste entfernt.


   // Spezielle Anfangs- und Endmethoden.

   void addhead(node *n);    // Fgt einen Knoten am Anfang einer Liste ein.
   void addtail(node *n);    // Fgt einen Knoten am Anfang einer Liste ein.

   node *remhead(void);      // Nimmt den ersten Knoten aus der Liste und
			     // gibt ihn zurck.
   node *remtail(void);      // Nimmt den letzten Knoten aus der Liste und
			     // gibt ihn zurck.


   // Spezielle Einfgekomandos

   void enqueuepri(node *n); // Fgt den Knoten aufgrund seiner
			     // Prioritt in die Liste ein. Ganz vorne stehen
			     // die Elemente mit kleiner Prioritt.
   void enqueuename(node *n); // s.o. Kriterium ist hier die
			     // alphabetische Reihenfolge der Namen.
   void enqueuepack(node *n); // s.o. Hier ist das Sortierkriterium
			     // der gepackte Name.

   // Spezielle Suchkommandos

   node *findpri(long p);    // Sucht die Liste nach einem Element mit
			     // der Prioritt 'p' ab, wenn ein solches
			     // existiert, wird es zurckgegeben, wenn nicht
			     // wird NULL zurckgegeben.
   node *findname(char *n);  // s.o. hier jedoch mit Namen
   node *findpack(unsigned long pa); // s.o. aber mit Pack


   // Sortiert kann die Liste auch werden

   void sortpri(void);       // Nach Prioritt sortieren
   void sortname(void);      // Nach Name sortieren
   void sortpack(void);      // Nach Pack sortieren


   // Ausgabekommandos

   void print(void);         // Gibt die Liste nach stdout aus.

   // Alles weitere nur unter AmigaOS vorhanden.
#ifdef amigados
 private:
   struct Window *displaywin; // Der Zeiger auf das Display Window.
			     // NULL, wenn das Window geschlossen.
   struct Gadget *g;	     // Zeiger auf das ListGadget
   struct Gadget *glist;     // Die Gadgetliste fr GadTools.
   struct Menu	 *menu;      // Die Menues des Fensters
   struct List execlist;     // Die Daten des ScrollerGadget mssen in einer
			     // Liste abgelegt sein. Dies ist sie!

   int change;		     // Gibt an, ob die Liste gerade gendert
			     // wird (dann hat 'change' den Wert 1, sonst 0)

   // Alle folgenden Variablen sind nur dazu da, den Status eines ver-
   // steckten Fensters zu speichern, um dann bei einem 'reveal' wieder
   // an der richtigen Stelle mit der richtigen Gre und den richtigen
   // Daten zu erscheinen.
   int W_hide;		     // gibt an, ob das Fenster versteckt ist:
			     // 0 - nicht versteckt
			     // 1 - versteckt
   int W_xpos, W_ypos, W_height, W_width; // Achtung!! W_height und W_width
			     // sind in Anzahl der Pixel angegeben! (zum
			     // Unterschied im Aufruf von display().
   char *W_wintitle, *W_scrtitle;
   struct MsgPort *W_mp;     // Der shard-Port.
   dsply	   W_ds;     // Anzeigemodus.
   char 	  *W_format; // Format der Liste im Fenster.

   // Die drei nachfolgenden Funktionen nehmen eine besondere Bedeutung ein.
   // Sie funktionieren nur auf dem Amiga, da die Intuition-Oberflche zum
   // Anzeigen und zur Auswahl eines Listenelements herangezogen wird.
   // Die Parameter: Alle Paremeter, bis auf 'up' und 'msg' sind optional.
   //
   // ds       readonly -> Kann nur angeschaut werden
   //	       readwrite -> Kann angesehen und ausgewhlt werden
   //
   // format   gibt an, welche Paramter in welcher Reihenfolge in dem
   //	       Display erscheinen.
   //	       p - Prioritt (mit C-Format "%6d")
   //	       a - Packed als Zahl (mit C-Format "%08lx")
   //	       i - Packed als Text (mit C-Format "%4s")
   //	       n - Name (mit C-Format "%s")
   //	       Es werden nur diese Buchstaben erwartet, alle anderen Angaben
   //	       werden berlesen.
   // width    Geben die Breite bzw. die Hhe des Feldes an, in dem die
   // height   Liste dargestellt wird. (Anzahl der !! Zeichen !!)
   //
   // xpos     Gibt die Stelle an, an der das Fenster erscheint.
   // ypos     (Werte in !! Pixel !!)
   //
   // wintitle Der Titel des Windows und der Screen.
   // scrtitle
   //
   // up       Den ShardPort, also den Port, ber den das Fenster die
   //	       Messages bekommt. Ist dieser angegeben, wird KEIN eigener
   //	       MessagePost geffnet
   //
   // msg      Eine IntuiMessage, die ber den ShardPort gekommen ist.
   //
   // Die Funktionen im einzelnen:
   // - Die erste Funktion ffnet ein Fenster, zeigt die Daten der Liste
   //	darin an, und wartet je nach Wert von 'ds' entweder darauf, da das
   //	Fenster geschlossen wird (falls ds==readonly) oder darauf, da ein
   //	Listenelement ausgewhlt wird oder das Fenster geschlossen wird (falls
   //	ds==readwrite). Falls das Fenster geschlossen wird, ohne da ein
   //	Listenelement ausgewhlt wurde, wird NULL zurckgegeben, ansonsten
   //	ein Zeiger auf dieses Element.
   // - Die zweite Funktion ffnet das Fenster und stellt die Daten dar,
   //	wartet aber NICHT auf eine Meldung von dem/der BenutzerIn sondern
   //	springt sofort zurck.
   // - Die Auswahl bernimmt dann die Funktion 'checkdis()', die eine Message
   //	vom ShardPort als Parameter besitzt.
   //	Wenn ein Listenelement ausgewhlt wurde, wird ein Zeiger darauf
   //	zurckgegeben.
   //	Wenn die Meldung nicht fr das Fenster bestimmt war, so bleibt die
   //	Meldung unangetastet und es wird zustzlich NULL zurckgegeben.
   //	War die Meldung fr dieses Fenster bestimmt, wird sie entsprechend
   //	behandelt, der WindowZeiger in der Meldung auf NULL gesetzt und NULL
   //	zurckgegeben.
   //	  Meldung	    |	  Aktion
   //	---------------------------------------------------------------------
   //	Fenster schliessen  |  Das Fenster wird 'versteckt' (hide)
   // - 'hide()' versteckt das Fenster explizit. Dabei bleiben interne
   //	Datenstrukturen erhalten und die Position des Fensters wird
   //	gespeichert. (Kann auch aufgerufen werden, wenn Fenster schon
   //	versteckt ist.)
   // - 'reveal()' ffnet ein verstecktes Window.
   // - 'closedis()' schliet ein geffnetes oder verstecktes Window und gibt
   //	smtliche Recourcen wieder frei.
 private:
   void doformat(char *buffer, char *format, node *n);
   int	domsg(struct IntuiMessage *msg);
   void freeexeclist(void);
   void makeexeclist(char *format);
   node *num2node(int num);
   char *pack2string(unsigned long pa);

 public:
   node *display(dsply ds=readonly, char *format="n",
		 int width=16, int height=16,
		 int xpos=50, int ypos=20,
		 char *wintitle="Display List", char *scrtitle="Display");
   int	display(struct MsgPort *up,
		struct Menu *menu=NULL,
		 dsply ds=readonly, char *format="n",
		 int width=16, int height=16,
		 int xpos=50, int ypos=20,
		 char *wintitle="Display List", char *scrtitle="Display");
   node *checkdis(struct IntuiMessage *msg);
   void hide(void);
   int	reveal(void);
   void closedis(void);
   // Und Funktionen, die es erlauben whrend das Window angezeigt wird die
   // Liste zu ndern.
   void start_change(void);
   void end_change(void);

   struct MsgPort *getmp(void);
   struct Menu *getmenu(void);
   struct Window *getwin(void);
#endif

};

#endif
